@@ -1,6 +1,6 @@
 /*  vdagentd.c vdagentd (daemon) code
 
-    Copyright 2010 Red Hat, Inc.
+    Copyright 2010-2012 Red Hat, Inc.
 
     Red Hat Authors:
     Hans de Goede <hdegoede@redhat.com>
@@ -38,13 +38,16 @@
 #include "vdagentd-proto.h"
 #include "vdagentd-proto-strings.h"
 #include "vdagentd-uinput.h"
+#include "vdagentd-xorg-conf.h"
 #include "vdagent-virtio-port.h"
-#include "console-kit.h"
+#include "session-info.h"
 
 struct agent_data {
     char *session;
     int width;
     int height;
+    struct vdagentd_guest_xorg_resolution *screen_info;
+    int screen_count;
 };
 
 /* variables */
@@ -55,14 +58,14 @@ static const char *uinput_device = "/dev/uinput";
 static int debug = 0;
 static struct udscs_server *server = NULL;
 static struct vdagent_virtio_port *virtio_port = NULL;
-#ifdef HAVE_CONSOLE_KIT
-static struct console_kit *console_kit = NULL;
+#ifdef HAVE_SESSION_INFO
+static struct session_info *session_info = NULL;
 #endif
 static struct vdagentd_uinput *uinput = NULL;
 static VDAgentMonitorsConfig *mon_config = NULL;
 static uint32_t *capabilities = NULL;
 static int capabilities_size = 0;
-#ifdef HAVE_CONSOLE_KIT
+#ifdef HAVE_SESSION_INFO
 static const char *active_session = NULL;
 #else
 static unsigned int session_count = 0;
@@ -116,6 +119,8 @@ static void do_client_monitors(struct vdagent_virtio_port *vport, int port_nr,
         return;
     }
 
+    vdagentd_write_xorg_conf(new_monitors, logfile);
+
     if (!mon_config ||
             mon_config->num_of_monitors != new_monitors->num_of_monitors) {
         free(mon_config);
@@ -238,6 +243,8 @@ int virtio_port_read_complete(
                 uinput = vdagentd_uinput_create(uinput_device,
                                                 agent_data->width,
                                                 agent_data->height,
+                                                agent_data->screen_info,
+                                                agent_data->screen_count,
                                                 logfile, debug > 1);
             if (!uinput) {
                 fprintf(logfile, "Fatal uinput error\n");
@@ -394,15 +401,20 @@ static void check_xorg_resolution(void)
 {
     struct agent_data *agent_data = udscs_get_user_data(active_session_conn);
 
-    if (agent_data && agent_data->width) {
+    if (agent_data && agent_data->screen_info) {
         if (!uinput)
             uinput = vdagentd_uinput_create(uinput_device,
                                             agent_data->width,
                                             agent_data->height,
+                                            agent_data->screen_info,
+                                            agent_data->screen_count,
                                             logfile, debug > 1);
         else
-            vdagentd_uinput_update_size(&uinput, agent_data->width,
-                                        agent_data->height);
+            vdagentd_uinput_update_size(&uinput,
+                                        agent_data->width,
+                                        agent_data->height,
+                                        agent_data->screen_info,
+                                        agent_data->screen_count);
         if (!uinput) {
             fprintf(logfile, "Fatal uinput error\n");
             retval = 1;
@@ -425,7 +437,9 @@ static void check_xorg_resolution(void)
             send_capabilities(virtio_port, 1);
         }
     } else {
+#ifndef WITH_STATIC_UINPUT
         vdagentd_uinput_destroy(&uinput);
+#endif
         if (virtio_port) {
             vdagent_virtio_port_flush(&virtio_port);
             vdagent_virtio_port_destroy(&virtio_port);
@@ -434,7 +448,7 @@ static void check_xorg_resolution(void)
     }
 }
 
-#ifdef HAVE_CONSOLE_KIT
+#ifdef HAVE_SESSION_INFO
 static int connection_matches_active_session(struct udscs_connection **connp,
     void *priv)
 {
@@ -470,9 +484,9 @@ void update_active_session_connection(void)
     struct udscs_connection *new_conn = NULL;
     int n;
 
-#ifdef HAVE_CONSOLE_KIT
+#ifdef HAVE_SESSION_INFO
     if (!active_session)
-        active_session = console_kit_get_active_session(console_kit);
+        active_session = session_info_get_active_session(session_info);
 
     n = udscs_server_for_all_clients(server, connection_matches_active_session,
                                      (void*)&new_conn);
@@ -501,9 +515,9 @@ void agent_connect(struct udscs_connection *conn)
         udscs_destroy_connection(&conn);
         return;
     }
-#ifdef HAVE_CONSOLE_KIT
+#ifdef HAVE_SESSION_INFO
     pid = udscs_get_peer_cred(conn).pid;
-    agent_data->session = console_kit_session_for_pid(console_kit, pid);
+    agent_data->session = session_info_session_for_pid(session_info, pid);
 #else
     session_count++;
     if (session_count == 1) {
@@ -535,12 +549,17 @@ void agent_disconnect(struct udscs_connection *conn)
 {
     struct agent_data *agent_data = udscs_get_user_data(conn);
 
+#ifndef HAVE_SESSION_INFO
+    if (conn == active_session_conn)
+        active_session_conn = NULL;
+#endif
+
     free(agent_data->session);
     agent_data->session = NULL;
     update_active_session_connection();
 
     free(agent_data);
-#ifndef HAVE_CONSOLE_KIT
+#ifndef HAVE_SESSION_INFO
     session_count--;
 #endif
 }
@@ -552,18 +571,38 @@ void agent_read_complete(struct udscs_connection **connp,
 
     switch (header->type) {
     case VDAGENTD_GUEST_XORG_RESOLUTION: {
-        struct vdagentd_guest_xorg_resolution *res =
-            (struct vdagentd_guest_xorg_resolution *)data;
+        struct vdagentd_guest_xorg_resolution *res;
+        int n = header->size / sizeof(*res);
 
-        if (header->size != sizeof(*res)) {
-            fprintf(logfile,
-                    "guest xorg resolution message has wrong size, disconnecting agent\n");
-            udscs_destroy_connection(connp);
+        /* Detect older version session agent, but don't disconnect, as
+           that stops it from getting the VDAGENTD_VERSION message, and then
+           it will never re-exec the new version... */
+        if (header->arg1 == 0 && header->arg2 == 0) {
+            fprintf(logfile, "got old session agent xorg resolution message, ignoring\n");
+            free(data);
             return;
         }
 
-        agent_data->width  = res->width;
-        agent_data->height = res->height;
+        if (header->size != n * sizeof(*res)) {
+            fprintf(logfile,
+                    "guest xorg resolution message has wrong size, disconnecting agent\n");
+            udscs_destroy_connection(connp);
+            free(data);
+            return;
+        }
+
+        free(agent_data->screen_info);
+        res = malloc(n * sizeof(*res));
+        if (!res) {
+            fprintf(logfile, "out of memory allocating screen info\n");
+            n = 0;
+        }
+        memcpy(res, data, n * sizeof(*res));
+        agent_data->width  = header->arg1;
+        agent_data->height = header->arg2;
+        agent_data->screen_info  = res;
+        agent_data->screen_count = n;
+
         check_xorg_resolution();
         break;
     }
@@ -573,6 +612,7 @@ void agent_read_complete(struct udscs_connection **connp,
     case VDAGENTD_CLIPBOARD_RELEASE:
         if (do_agent_clipboard(*connp, header, data)) {
             udscs_destroy_connection(connp);
+            free(data);
             return;
         }
         break;
@@ -608,7 +648,7 @@ void daemonize(void)
     case 0:
         close(0); close(1); close(2);
         setsid();
-        x = open("/dev/null", O_RDWR); dup(x); dup(x);
+        x = open("/dev/null", O_RDWR); x = dup(x); x = dup(x);
         pidfile = fopen(pidfilename, "w");
         if (pidfile) {
             fprintf(pidfile, "%d\n", (int)getpid());
@@ -640,8 +680,8 @@ void main_loop(void)
         if (n >= nfds)
             nfds = n + 1;
 
-#ifdef HAVE_CONSOLE_KIT
-        ck_fd = console_kit_get_fd(console_kit);
+#ifdef HAVE_SESSION_INFO
+        ck_fd = session_info_get_fd(session_info);
         FD_SET(ck_fd, &readfds);
         if (ck_fd >= nfds)
             nfds = ck_fd + 1;
@@ -675,9 +715,9 @@ void main_loop(void)
             }
         }
 
-#ifdef HAVE_CONSOLE_KIT
+#ifdef HAVE_SESSION_INFO
         if (FD_ISSET(ck_fd, &readfds)) {
-            active_session = console_kit_get_active_session(console_kit);
+            active_session = session_info_get_active_session(session_info);
             update_active_session_connection();
         }
 #endif
@@ -763,10 +803,22 @@ int main(int argc, char *argv[])
     if (do_daemonize)
         daemonize();
 
-#ifdef HAVE_CONSOLE_KIT
-    console_kit = console_kit_create(logfile);
-    if (!console_kit) {
-        fprintf(logfile, "Fatal could not connect to console kit\n");
+#ifdef WITH_STATIC_UINPUT
+    uinput = vdagentd_uinput_create(uinput_device, 1024, 768, NULL, 0,
+                                    logfile, debug > 1);
+    if (!uinput) {
+        udscs_destroy_server(server);
+        if (logfile != stderr)
+            fclose(logfile);
+        return 1;
+    }
+#endif
+
+#ifdef HAVE_SESSION_INFO
+    session_info = session_info_create(logfile, debug);
+    if (!session_info) {
+        fprintf(logfile, "Fatal could not get session information\n");
+        vdagentd_uinput_destroy(&uinput);
         udscs_destroy_server(server);
         if (logfile != stderr)
             fclose(logfile);
@@ -781,8 +833,8 @@ int main(int argc, char *argv[])
     vdagentd_uinput_destroy(&uinput);
     vdagent_virtio_port_flush(&virtio_port);
     vdagent_virtio_port_destroy(&virtio_port);
-#ifdef HAVE_CONSOLE_KIT
-    console_kit_destroy(console_kit);
+#ifdef HAVE_SESSION_INFO
+    session_info_destroy(session_info);
 #endif
     udscs_destroy_server(server);
     if (unlink(VDAGENTD_SOCKET) != 0)
